@@ -67,12 +67,12 @@ static int on_headers_complete(http_parser* parser)
     return skip_body;
 }
 
-static int on_header_field(http_parser* parser, const char *at, size_t length)
+static int on_http_data_cb(http_parser* parser, const char *at, size_t length, const char * python_cb)
 {
     int fail = 0;
     PyObject* self = (PyObject*)parser->data;
-    if (PyObject_HasAttrString(self, "_on_header_field")) {
-        PyObject* callable = PyObject_GetAttrString(self, "_on_header_field");
+    if (PyObject_HasAttrString(self, python_cb)) {
+        PyObject* callable = PyObject_GetAttrString(self, python_cb);
         PyObject* args = Py_BuildValue("(s#)", at, length);
         PyObject* result = PyObject_CallObject(callable, args);
         PyObject* exception = PyErr_Occurred();
@@ -86,29 +86,22 @@ static int on_header_field(http_parser* parser, const char *at, size_t length)
         Py_DECREF(callable);
         Py_DECREF(args);
     }
-    return fail;
+    return fail;    
+}
+
+static int on_status(http_parser* parser, const char *at, size_t length)
+{
+    return on_http_data_cb(parser, at, length, "_on_status");
+}
+
+static int on_header_field(http_parser* parser, const char *at, size_t length)
+{
+    return on_http_data_cb(parser, at, length, "_on_header_field");
 }
 
 static int on_header_value(http_parser* parser, const char *at, size_t length)
 {
-    int fail = 0;
-    PyObject* self = (PyObject*)parser->data;
-    if (PyObject_HasAttrString(self, "_on_header_value")) {
-        PyObject* callable = PyObject_GetAttrString(self, "_on_header_value");
-        PyObject* args = Py_BuildValue("(s#)", at, length);
-        PyObject* result = PyObject_CallObject(callable, args);
-        PyObject* exception = PyErr_Occurred();
-        if (exception != NULL) {
-            fail = 1;
-        } else {
-            if (PyObject_IsTrue(result))
-                fail = 1;
-        }
-        Py_XDECREF(result);
-        Py_DECREF(callable);
-        Py_DECREF(args);
-    }
-    return fail;
+    return on_http_data_cb(parser, at, length, "_on_header_value");
 }
 
 static int on_body(http_parser* parser, const char *at, size_t length)
@@ -137,6 +130,7 @@ static int on_body(http_parser* parser, const char *at, size_t length)
 static http_parser_settings _parser_settings = {
     on_message_begin,
     NULL, // on_url
+    on_status,
     on_header_field,
     on_header_value,
     on_headers_complete,
@@ -183,19 +177,22 @@ PyHTTPResponseParser_feed(PyHTTPResponseParser *self, PyObject* args)
 {
     char* buf = NULL;
     Py_ssize_t buf_len;
-    int succeed = PyArg_ParseTuple(args, "t#", &buf, &buf_len);
+    int succeed = PyArg_ParseTuple(args, "s#", &buf, &buf_len);
     /* cast Py_ssize_t signed integer to unsigned */
     size_t unsigned_buf_len = buf_len + size_t_MAX + 1;
     if (succeed) {
+        size_t nread;
+        PyObject * exception;
+
         /* in case feed is called again after an error occured */
         if (self->parser->http_errno != HPE_OK)
             return set_parser_exception(self->parser);
 
-        size_t nread = http_parser_execute(self->parser,
+        nread = http_parser_execute(self->parser,
                 &_parser_settings, buf, unsigned_buf_len);
 
         /* Exception in callbacks */
-        PyObject * exception = PyErr_Occurred();
+        exception = PyErr_Occurred();
         if (exception != NULL)
             return NULL;
 
@@ -216,12 +213,21 @@ PyHTTPResponseParser_parser_failed(PyHTTPResponseParser* self)
     Py_RETURN_FALSE;
 }
 
+#if PY_MAJOR_VERSION >= 3
+static PyObject*
+PyHTTPResponseParser_get_http_version(PyHTTPResponseParser *self)
+{
+    return PyUnicode_FromFormat("HTTP/%u.%u", self->parser->http_major,
+        self->parser->http_minor);
+}
+#else
 static PyObject*
 PyHTTPResponseParser_get_http_version(PyHTTPResponseParser *self)
 {
     return PyString_FromFormat("HTTP/%u.%u", self->parser->http_major,
         self->parser->http_minor);
 }
+#endif
 
 static PyObject*
 PyHTTPResponseParser_get_remaining_content_length(PyHTTPResponseParser *self)
@@ -275,8 +281,7 @@ static PyMethodDef PyHTTPResponseParser_methods[] = {
 };
 
 static PyTypeObject HTTPParserType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "HTTPResponseParser",      /*tp_name*/
     sizeof(PyHTTPResponseParser),      /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -320,30 +325,58 @@ static PyMethodDef module_methods[] = {
     {NULL}  /* Sentinel */
 };
 
-#ifndef PyMODINIT_FUNC /* declarations for DLL import/export */
-#define PyMODINIT_FUNC void
-#endif
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_parser",
+        "HTTP Parser from nginx/Joyent.",
+        0,
+        module_methods,
+        NULL,
+        NULL,
+        NULL,
+        NULL
+};
+
+#define INITERROR return NULL
 PyMODINIT_FUNC
+PyInit__parser(void)
+
+#else
+#define INITERROR return
+void
 init_parser(void)
+#endif
 {
-    PyObject* module;
+    PyObject *module, *httplib, *HTTPException;
 
     if (PyType_Ready(&HTTPParserType) < 0)
-        return;
+        INITERROR;
 
+    #if PY_MAJOR_VERSION >= 3
+    module = PyModule_Create(&moduledef);
+    #else
     module = Py_InitModule3("_parser", module_methods,
                        "HTTP Parser from nginx/Joyent.");
+    #endif
 
     Py_INCREF(&HTTPParserType);
     PyModule_AddObject(module, "HTTPResponseParser", (PyObject *)&HTTPParserType);
 
-    PyObject* httplib = PyImport_ImportModule("httplib");
-    PyObject* HTTPException = PyObject_GetAttrString(httplib, "HTTPException");
+    #if PY_MAJOR_VERSION >= 3
+    httplib = PyImport_ImportModule("http.client");
+    #else
+    httplib = PyImport_ImportModule("httplib");
+    #endif
+    HTTPException = PyObject_GetAttrString(httplib, "HTTPException");
 
     PyExc_HTTPParseError = PyErr_NewException(
             "_parser.HTTPParseError", HTTPException, NULL);
     Py_INCREF(PyExc_HTTPParseError);
     PyModule_AddObject(module, "HTTPParseError", PyExc_HTTPParseError);
+    #if PY_MAJOR_VERSION >= 3
+    return  module;
+    #endif
 }
 
 #undef PY_SSIZE_T_CLEAN
